from django.conf.urls.defaults import patterns, url

urlpatterns = patterns(
    'preferences.views',
    url(r'^preferences/(?P<model_name>[\w-]+)/$', 'singleton_redirect', name='preferences_singelton_redirect'),
    url(r'^preferences/(?P<model_name>[\w-]+)/add/$', 'singleton_redirect', name='preferences_singelton_redirect'),
)



from django.db import models

import preferences
from preferences.managers import SingletonManager

class Preferences(models.Model):
    singleton = SingletonManager()

def preferences_class_prepared(sender, *args, **kwargs):
    cls = sender
   
    if issubclass(cls, Preferences):
        # add singleton manager to subclasses
        cls.add_to_class('singleton', SingletonManager())
        # add property for preferences object to preferences.preferences
        setattr(preferences.Preferences, cls._meta.object_name, property(lambda x: cls.singleton.get()))

models.signals.class_prepared.connect(preferences_class_prepared)



from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.shortcuts import redirect

def singleton_redirect(request, model_name):
    cls = ContentType.objects.get(app_label="preferences", model=model_name).model_class()
    obj = cls.singleton.get()
    url = reverse('admin:preferences_%s_change' % model_name, args=(obj.id,))
    return redirect(url)



from django.db import models
from django.db.utils import DatabaseError

class SingletonManager(models.Manager):
    def get_query_set(self):
        # get and return the first options object
        queryset = super(SingletonManager, self).get_query_set()
        try:
            queryset.get()
        except self.model.DoesNotExist:
            obj = self.model()
            obj.save()
           
        return queryset
